/*
Copyright (c) 2015, Patrick Weltevrede
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.

3. Neither the name of the copyright holder nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <string.h>
#include <stdlib.h>
#include "psrsalsa.h"
#define MaxNrJumps 100
int main(int argc, char **argv)
{
  int index, firstfiletoopen, curpanelnrx, curpanelnry, nokeypresses, showtotPol, nopaswing;
  int deviceID, nrpanelsx, nrpanelsy, nxsub, nysub_pa, nysub_ell;
  int titleset, dashed, datalinewidth, noylabel, overlayPA, overlayFine, nrJumps;
  int outline, outline_color, xunit_type, nowedge;
  long i;
  float sigma_limit, titlech, boxch, labelch;
  float xsize, ysize, ysizepa, xtick, ytick_pa, ytick_ell, plotI1, plotI2;
  float plotl1, plotl2, plotp1, plotp2, plotp1_padist, plotp2_padist, PAoffset, loffset;
  float overlayalpha, overlaybeta, overlaypa0, overlayl0;
  float jump_longitude[MaxNrJumps], jump_offset[MaxNrJumps];
  float padist_saturize, elldist_saturize;
  char *filename_ptr, title[1000], PlotDevice[100];
  char *label_pa = "PA [deg]";
  char *label_ell = "\\gx [deg]";
  char *label_I = "I,Linear,V";
  char *label_x = "Pulse longitude [deg]";
  int label_x_set, showEll;
  datafile_definition datain;
  psrsalsaApplication application;
  pgplot_options_definition pgplot_options;
  pgplot_clear_options(&pgplot_options);
  datafile_definition padist_data, elldist_data;
  int padist_filename_index, elldist_filename_index, padist_overlayavpa, padist_paswing;
  initApplication(&application, "ppolFig", "[options] inputfile(s)");
  application.switch_verbose = 1;
  application.switch_debug = 1;
  application.switch_header = 1;
  application.switch_headerlist = 1;
  application.switch_nocounters = 1;
  application.switch_filelist = 1;
  application.switch_iformat = 1;
  curpanelnrx = 0;
  curpanelnry = 0;
  nokeypresses = 0;
  showtotPol = 0;
  strcpy(PlotDevice, "?");
  nrpanelsx = 1;
  nrpanelsy = 1;
  xsize = 0.8;
  ysize = 1.0;
  ysizepa = 1.0;
  xtick = 0.0;
  ytick_pa = 0.0;
  ytick_ell = 0.0;
  nxsub = 0;
  nysub_pa = 0;
  nysub_ell = 0;
  sigma_limit = 3.0;
  titleset = 0;
  title[0] = 0;
  plotl1 = 0;
  plotl2 = 360;
  plotp1 = -180;
  plotp2 = 180;
  plotp1_padist = -90;
  plotp2_padist = 90;
  plotI1 = plotI2 = 0;
  PAoffset = 0;
  loffset = 0;
  datalinewidth = 1;
  dashed = 0;
  noylabel = 0;
  overlayPA = 0;
  overlayFine = 1;
  outline = -1;
  nrJumps = 0;
  nopaswing = 0;
  titlech = 1;
  boxch = 1;
  labelch = 1;
  xunit_type = 0;
  nowedge = 0;
  padist_saturize = 1.0;
  elldist_saturize = 1.0;
  label_x_set = 0;
  showEll = 0;
  padist_filename_index = 0;
  elldist_filename_index = 0;
  padist_overlayavpa = 0;
  padist_paswing = 0;
  if(argc < 2) {
    printf("Program to plot output generated by ppol.\n\n");
    printApplicationHelp(&application);
    fprintf(stdout, "Other options affecting the reading of the input file:\n");
    fprintf(stdout, "-loffset           Shift longitudes by this amount.\n");
    fprintf(stdout, "-paoffset          Add this angle to PA (in degrees).\n");
    fprintf(stdout, "-sigma             Set sigma limit on L required for PA calculation [def=%.1f].\n", sigma_limit);
    fprintf(stdout, "\nOther options affecting the plotting:\n");
    fprintf(stdout, "-1             Only plot PA-swing once (equivalent to -yrange \"0 180\").\n");
    fprintf(stdout, "-box_fmt       \"characterheight line_width font\" (default being \"%.1f %d %d\").\n", boxch, pgplot_options.box.box_lw, pgplot_options.box.box_f);
    fprintf(stdout, "-dash          Make L and V profiles dashed/dotted.\n");
    fprintf(stdout, "-device        Specify plotting device final plot.\n");
    fprintf(stdout, "-ell           Show the ellipticity angle.\n");
    fprintf(stdout, "-herrorbar     \"xleft xcentre xright y SizeOfMarkers lineWidth colourIndex\"\n");
    fprintf(stdout, "               Draw a horizontal errorbar. The position y is normalised\n");
    fprintf(stdout, "               between 0 and 1.\n");
    fprintf(stdout, "-herrorbarpa   Identical to -herrorbar option, but now for the PA panel,\n");
    fprintf(stdout, "               except that y is in degrees rather than being normalized.\n");
    fprintf(stdout, "-label_fmt       \"characterheight line_width font\" (default being \"%.1f %d %d\").\n", labelch, pgplot_options.box.label_lw, pgplot_options.box.label_f);
    fprintf(stdout, "-labelell \"..\" Set the label to be used for ellipticity.\n");
    fprintf(stdout, "-labelpa \"...\" Set the label to be used for PA.\n");
    fprintf(stdout, "-labelI  \"...\" Set the label to be used for intensities.\n");
    fprintf(stdout, "-lw            Set linewidth used to plot data [def=%d].\n", datalinewidth);
    fprintf(stdout, "-N             \"nrx nry\" Create nrx by nry panels, rather than a single plot\n");
    fprintf(stdout, "               per page\n");
    fprintf(stdout, "-nokeypress    Do not wait for key presses to go to next page in output plot\n");
    fprintf(stdout, "               (by default this shouldn't happen if plotting to a file)\n");
    fprintf(stdout, "-nowedge       Do not indicate color scale of PA/ellipticity distribution.\n");
    fprintf(stdout, "-noylabel      Disable labeling and numbers along y-axis.\n");
    fprintf(stdout, "-nopaswing     Do not produce the PA-swing panel.\n");
    fprintf(stdout, "-opm           Put OPM at this longitude and with this amount of degrees when\n");
    fprintf(stdout, "               using -paswing. You can use this option multiple times.\n");
    fprintf(stdout, "-outline       \"lw color\" Makes text of -text option appear in outline\n");
    fprintf(stdout, "-padist        Specify filename of the PA distribution to plot underneath the\n");
    fprintf(stdout, "               polarized profile.\n");
    fprintf(stdout, "-padist_sat f  If f > 1, the PA distribution will be saturated, revealing\n");
    fprintf(stdout, "               weaker structures.\n");
    fprintf(stdout, "-elldist       Specify filename of the ellipticity angle distribution to plot\n");
    fprintf(stdout, "               underneath the polarized profile.\n");
    fprintf(stdout, "-elldist_sat f As -padist_sat, but for ellipticity distribution.\n");
    fprintf(stdout, "-paswing       \"alpha beta pa0 l0\" Overlay PA-swing with these parameters.\n");
    fprintf(stdout, "-text          \"x y ch lw font color\" \"text\" plots text at this position in\n");
    fprintf(stdout, "               the top panel. See -textkeywords for possible keywords to use.\n");
    fprintf(stdout, "-text_pa       As -text, but now relative to the PA-swing panel.\n");
    fprintf(stdout, "-text_ell      As -text, but now relative to the ellipticity swing panel.\n");
    fprintf(stdout, "-text_padist   As -text, but now relative to the PA distribution panel.\n");
    fprintf(stdout, "-text_elldist  As -text, but now relative to the ellipticity distribution.\n");
    fprintf(stdout, "-textkeywords  Lists to keywords you can use in the -text and -title options.\n");
    fprintf(stdout, "-title \"...\"   Set title (default is file name). The title supports keywords\n");
    fprintf(stdout, "               listed by the -textkeywords option.\n");
    fprintf(stdout, "-title_fmt     \"characterheight line_width font\" (default being \"%.1f %d %d\").\n", titlech, pgplot_options.box.title_lw, pgplot_options.box.title_f);
    fprintf(stdout, "-totpol        Show the total amount of polarization.\n");
    fprintf(stdout, "-xrange        Specify longitude range covered in the plot in degrees.\n");
    fprintf(stdout, "-xrange_phase  Specify longitude range covered in the plot in phase.\n");
    fprintf(stdout, "-yrange        Specify PA-range covered in PA-swing plot.\n");
    fprintf(stdout, "-yrangepadist  Specify PA-range covered in PA distribution plot.\n");
    fprintf(stdout, "-xsize         Set xsize of the plot [def=%.1f].\n", xsize);
    fprintf(stdout, "-ysize         Set ysize of the plot [def=%.1f].\n", ysize);
    fprintf(stdout, "-ysizepa       Set relative ysize of PA plot [def=%.1f].\n", ysizepa);
    fprintf(stdout, "-xticks        \"XTICK NXSUB\"  Adjust PGPLOT tickmarks [def=\"%.0f %d\"].\n", xtick, nxsub);
    fprintf(stdout, "-yticks_pa     As -xticks, but for the PA-swing panel [def=\"%.0f %d\"].\n", ytick_pa, nysub_pa);
    fprintf(stdout, "-yticks_ell     As -xticks, but for the ellipticity-swing panel [def=\"%.0f %d\"].\n", ytick_ell, nysub_ell);
    printf("\n");
    printf("Please use the appropriate citation when using results of this software in your publications:\n\n");
    printf("More information about fitting position-angle swings and using the beam-width information can be found in:\n");
    printf(" - 	Rookyard et al. 2015, MNRAS, 446, 3367\n\n");
    printCitationInfo();
    terminateApplication(&application);
    return 0;
  }else if(argc >= 2) {
    for(i = 1; i < argc; i++) {
      index = i;
      if(processCommandLine(&application, argc, argv, &index)) {
 i = index;
      }else if(strcmp(argv[i], "-device") == 0) {
 strcpy(PlotDevice,argv[i+1]);
        i++;
      }else if(strcmp(argv[i], "-paoffset") == 0) {
 if(parse_command_string(application.verbose_state, argc, argv, i+1, 0, -1, "%f", &PAoffset, NULL) == 0) {
   printerror(application.verbose_state.debug, "ERROR ppolFig: Cannot parse '%s' option.", argv[i]);
   return 0;
 }
 i++;
      }else if(strcmp(argv[i], "-loffset") == 0) {
 if(parse_command_string(application.verbose_state, argc, argv, i+1, 0, -1, "%f", &loffset, NULL) == 0) {
   printerror(application.verbose_state.debug, "ERROR ppolFig: Cannot parse '%s' option.", argv[i]);
   return 0;
 }
 i++;
      }else if(strcmp(argv[i], "-nokeypress") == 0) {
 nokeypresses = 1;
      }else if(strcmp(argv[i], "-totPol") == 0 || strcmp(argv[i], "-totpol") == 0) {
 showtotPol = 1;
      }else if(strcmp(argv[i], "-ell") == 0 || strcmp(argv[i], "-Ell") == 0 || strcmp(argv[i], "-ellipticity") == 0) {
 showEll = 1;
      }else if(strcmp(argv[i], "-nowedge") == 0) {
 nowedge = 1;
      }else if(strcmp(argv[i], "-dash") == 0) {
 dashed = 1;
      }else if(strcmp(argv[i], "-noylabel") == 0) {
 noylabel = 1;
      }else if(strcmp(argv[i], "-labelpa") == 0) {
 label_pa = argv[i+1];
 i++;
      }else if(strcmp(argv[i], "-labelell") == 0) {
 label_ell = argv[i+1];
 i++;
      }else if(strcmp(argv[i], "-labelI") == 0) {
 label_I = argv[i+1];
 i++;
      }else if(strcmp(argv[i], "-labelx") == 0) {
 label_x = argv[i+1];
 label_x_set = 1;
 i++;
      }else if(strcmp(argv[i], "-N") == 0) {
 if(parse_command_string(application.verbose_state, argc, argv, i+1, 0, -1, "%d %d", &nrpanelsx, &nrpanelsy, NULL) == 0) {
   printerror(application.verbose_state.debug, "ERROR ppolFig: Cannot parse '%s' option.", argv[i]);
   return 0;
 }
        i++;
      }else if(strcmp(argv[i], "-xsize") == 0) {
 if(parse_command_string(application.verbose_state, argc, argv, i+1, 0, -1, "%f", &xsize, NULL) == 0) {
   printerror(application.verbose_state.debug, "ERROR ppolFig: Cannot parse '%s' option.", argv[i]);
   return 0;
 }
        i++;
      }else if(strcmp(argv[i], "-ysize") == 0) {
 if(parse_command_string(application.verbose_state, argc, argv, i+1, 0, -1, "%f", &ysize, NULL) == 0) {
   printerror(application.verbose_state.debug, "ERROR ppolFig: Cannot parse '%s' option.", argv[i]);
   return 0;
 }
        i++;
      }else if(strcmp(argv[i], "-sigma") == 0) {
 if(parse_command_string(application.verbose_state, argc, argv, i+1, 0, -1, "%f", &sigma_limit, NULL) == 0) {
   printerror(application.verbose_state.debug, "ERROR ppolFig: Cannot parse '%s' option.", argv[i]);
   return 0;
 }
        i++;
      }else if(strcmp(argv[i], "-ysizepa") == 0) {
 if(parse_command_string(application.verbose_state, argc, argv, i+1, 0, -1, "%f", &ysizepa, NULL) == 0) {
   printerror(application.verbose_state.debug, "ERROR ppolFig: Cannot parse '%s' option.", argv[i]);
   return 0;
 }
        i++;
      }else if(strcmp(argv[i], "-title") == 0) {
 strcpy(title, argv[i+1]);
 titleset = 1;
        i++;
      }else if(strcmp(argv[i], "-title_fmt") == 0) {
 if(parse_command_string(application.verbose_state, argc, argv, i+1, 0, -1, "%f %d %d", &titlech, &pgplot_options.box.title_lw, &pgplot_options.box.title_f, NULL) == 0) {
   printerror(application.verbose_state.debug, "ERROR ppolFig: Cannot parse '%s' option.", argv[i]);
   return 0;
 }
        i++;
      }else if(strcmp(argv[i], "-box_fmt") == 0) {
 if(parse_command_string(application.verbose_state, argc, argv, i+1, 0, -1, "%f %d %d", &boxch, &pgplot_options.box.box_lw, &pgplot_options.box.box_f, NULL) == 0) {
   printerror(application.verbose_state.debug, "ERROR ppolFig: Cannot parse '%s' option.", argv[i]);
   return 0;
 }
        i++;
      }else if(strcmp(argv[i], "-label_fmt") == 0) {
 if(parse_command_string(application.verbose_state, argc, argv, i+1, 0, -1, "%f %d %d", &labelch, &pgplot_options.box.label_lw, &pgplot_options.box.label_f, NULL) == 0) {
   printerror(application.verbose_state.debug, "ERROR ppolFig: Cannot parse '%s' option.", argv[i]);
   return 0;
 }
        i++;
      }else if(strcmp(argv[i], "-padist") == 0) {
 padist_filename_index = i+1;
        i++;
      }else if(strcmp(argv[i], "-elldist") == 0) {
 elldist_filename_index = i+1;
        i++;
      }else if(strcmp(argv[i], "-padist_sat") == 0) {
 if(parse_command_string(application.verbose_state, argc, argv, i+1, 0, -1, "%f", &padist_saturize, NULL) == 0) {
   printerror(application.verbose_state.debug, "ERROR ppolFig: Cannot parse '%s' option.", argv[i]);
   return 0;
 }
        i++;
      }else if(strcmp(argv[i], "-elldist_sat") == 0) {
 if(parse_command_string(application.verbose_state, argc, argv, i+1, 0, -1, "%f", &elldist_saturize, NULL) == 0) {
   printerror(application.verbose_state.debug, "ERROR ppolFig: Cannot parse '%s' option.", argv[i]);
   return 0;
 }
        i++;
      }else if(strcmp(argv[i], "-nopaswing") == 0) {
 nopaswing = 1;
      }else if(strcmp(argv[i], "-lw") == 0) {
 if(parse_command_string(application.verbose_state, argc, argv, i+1, 0, -1, "%d", &datalinewidth, NULL) == 0) {
   printerror(application.verbose_state.debug, "ERROR ppolFig: Cannot parse '%s' option.", argv[i]);
   return 0;
 }
        i++;
      }else if(strcmp(argv[i], "-xrange") == 0 || strcmp(argv[i], "-xrange_phase") == 0) {
 if(parse_command_string(application.verbose_state, argc, argv, i+1, 0, -1, "%f %f", &plotl1, &plotl2, NULL) == 0) {
   printerror(application.verbose_state.debug, "ERROR ppolFig: Cannot parse '%s' option.", argv[i]);
   return 0;
 }
 if(strcmp(argv[i], "-xrange_phase") == 0) {
   xunit_type = 1;
   plotl1 *= 360.0;
   plotl2 *= 360.0;
   if(label_x_set == 0) {
     label_x = "Pulse phase";
   }
 }
        i++;
      }else if(strcmp(argv[i], "-xticks") == 0) {
 if(parse_command_string(application.verbose_state, argc, argv, i+1, 0, -1, "%f %d", &xtick, &nxsub, NULL) == 0) {
   printerror(application.verbose_state.debug, "ERROR ppolFig: Cannot parse '%s' option.", argv[i]);
   return 0;
 }
        i++;
      }else if(strcmp(argv[i], "-yticks_pa") == 0) {
 if(parse_command_string(application.verbose_state, argc, argv, i+1, 0, -1, "%f %d", &ytick_pa, &nysub_pa, NULL) == 0) {
   printerror(application.verbose_state.debug, "ERROR ppolFig: Cannot parse '%s' option.", argv[i]);
   return 0;
 }
        i++;
      }else if(strcmp(argv[i], "-yticks_ell") == 0) {
 if(parse_command_string(application.verbose_state, argc, argv, i+1, 0, -1, "%f %d", &ytick_ell, &nysub_ell, NULL) == 0) {
   printerror(application.verbose_state.debug, "ERROR ppolFig: Cannot parse '%s' option.", argv[i]);
   return 0;
 }
        i++;
      }else if(strcmp(argv[i], "-yrange") == 0) {
 if(parse_command_string(application.verbose_state, argc, argv, i+1, 0, -1, "%f %f", &plotp1, &plotp2, NULL) == 0) {
   printerror(application.verbose_state.debug, "ERROR ppolFig: Cannot parse '%s' option.", argv[i]);
   return 0;
 }
        i++;
      }else if(strcmp(argv[i], "-yrangepadist") == 0) {
 if(parse_command_string(application.verbose_state, argc, argv, i+1, 0, -1, "%f %f", &plotp1_padist, &plotp2_padist, NULL) == 0) {
   printerror(application.verbose_state.debug, "ERROR ppolFig: Cannot parse '%s' option.", argv[i]);
   return 0;
 }
        i++;
      }else if(strcmp(argv[i], "-yrange2") == 0) {
 if(parse_command_string(application.verbose_state, argc, argv, i+1, 0, -1, "%f %f", &plotI1, &plotI2, NULL) == 0) {
   printerror(application.verbose_state.debug, "ERROR ppolFig: Cannot parse '%s' option.", argv[i]);
   return 0;
 }
        i++;
      }else if(strcmp(argv[i], "-paswing") == 0) {
 if(parse_command_string(application.verbose_state, argc, argv, i+1, 0, -1, "%f %f %f %f", &overlayalpha, &overlaybeta, &overlaypa0, &overlayl0, NULL) == 0) {
   printerror(application.verbose_state.debug, "ERROR ppolFig: Cannot parse '%s' option.", argv[i]);
   return 0;
 }
 overlayPA = 1;
        i++;
      }else if(strcmp(argv[i], "-1") == 0) {
 plotp1 = 0.0;
 plotp2 = 180.0;
      }else if(strcmp(argv[i], "-text") == 0) {
 i += 2;
      }else if(strcmp(argv[i], "-text_pa") == 0) {
 i += 2;
      }else if(strcmp(argv[i], "-text_ell") == 0) {
 i += 2;
      }else if(strcmp(argv[i], "-text_padist") == 0) {
 i += 2;
      }else if(strcmp(argv[i], "-text_elldist") == 0) {
 i += 2;
      }else if(strcmp(argv[i], "-herrorbar") == 0) {
 i += 1;
      }else if(strcmp(argv[i], "-herrorbarpa") == 0 || strcmp(argv[i], "-herrorbar2") == 0) {
 i += 1;
      }else if(strcmp(argv[i], "-outline") == 0) {
 if(parse_command_string(application.verbose_state, argc, argv, i+1, 0, -1, "%d %d", &outline, &outline_color, NULL) == 0) {
   printerror(application.verbose_state.debug, "ERROR ppolFig: Cannot parse '%s' option.", argv[i]);
   return 0;
 }
 i++;
      }else if(strcmp(argv[i], "-opm") == 0) {
 if(parse_command_string(application.verbose_state, argc, argv, i+1, 0, -1, "%f %f", &jump_longitude[nrJumps], &jump_offset[nrJumps], NULL) == 0) {
   printerror(application.verbose_state.debug, "ERROR ppolFig: Cannot parse '%s' option.", argv[i]);
   return 0;
 }
 nrJumps++;
 if(nrJumps == MaxNrJumps) {
   fflush(stdout);
   printerror(application.verbose_state.debug, "ERROR ppolFig: Maximum number of %s options exceeded.", argv[i]);
   return 0;
 }
 i++;
      }else if(strcmp(argv[i], "-textkeywords") == 0) {
 str_list_replace_keys(0);
 terminateApplication(&application);
 return 0;
      }else {
 if(argv[i][0] == '-') {
   printerror(application.verbose_state.debug, "ppolFig: Unknown option: %s", argv[i]);
   printerror(application.verbose_state.debug, "\nRun ppolFig without command line arguments to show help");
   terminateApplication(&application);
   return 0;
 }else {
   if(applicationAddFilename(i, application.verbose_state) == 0)
     return 0;
 }
      }
    }
  }
  if(applicationFilenameList_checkConsecutive(argv, application.verbose_state) == 0) {
    return 0;
  }
  if(numberInApplicationFilenameList(&application, argv, application.verbose_state) == 0) {
    fflush(stdout);
    printerror(application.verbose_state.debug, "ERROR ppolFig: No files specified");
    return 0;
  }
  firstfiletoopen = 1;
  while((filename_ptr = getNextFilenameFromList(&application, argv, application.verbose_state)) != NULL) {
    if(firstfiletoopen == 0) {
      if(curpanelnrx == 0 && curpanelnry == 0) {
 if(nokeypresses == 0) {
   i = pgplot_device_type(PlotDevice, application.verbose_state);
   if(i < 3 || i > 10) {
     printf("Press a key in the terminal to continue\n");
     fflush(stdout);
     pgetch();
   }
 }
      }
    }
    if(!openPSRData(&datain, filename_ptr, application.iformat, 0, 1, 0, application.verbose_state))
      return 0;
    if(application.verbose_state.verbose) {
      fflush(stdout);
      printwarning(application.verbose_state.debug, "Input data contains %ld bins, %ld pulses, %ld polarizations and %ld frequencies.", (datain.NrBins), datain.NrSubints, (datain.NrPols), datain.NrFreqChan);
    }
    if(PSRDataHeader_parse_commandline(&datain, argc, argv, application.verbose_state) == 0)
      return 0;
    if(datain.poltype != POLTYPE_ILVPAdPA && datain.poltype != POLTYPE_PAdPA && datain.poltype != POLTYPE_ILVPAdPATEldEl) {
      fflush(stdout);
      printerror(application.verbose_state.debug, "ERROR ppolFig: File does not appear to contain PA data. ppolFig will not show data.");
      return 0;
    }
    if(datain.poltype == POLTYPE_PAdPA) {
      fflush(stdout);
      printerror(application.verbose_state.debug, "ERROR ppolFig: File does not appear to contain a pulse profile. ppolFig will not show data.");
      return 0;
    }
    if(datain.NrPols != 5 && datain.NrPols != 8) {
      fflush(stdout);
      printerror(application.verbose_state.debug, "ERROR ppolFig: When plotting previously calculated PA data, the number of polarization channels is expected to be 5 or 8, but it is %d", datain.NrPols);
      if(datain.NrPols == 2) {
 printerror(application.verbose_state.debug, "Note that the PPOLSHORT format can not be displayed with ppolFig.");
      }
      return 0;
    }
    float currentPanelScaling, currentPanelScalingx, currentPanelScalingy, viewport_left, viewport_right, viewport_top, viewport_bottom;
    currentPanelScaling = 1;
    viewport_left = 0.1;
    if(nrpanelsx > 1) {
      currentPanelScalingx = xsize/((nrpanelsx-1)*0.2 + nrpanelsx*xsize);
      currentPanelScaling = currentPanelScalingx;
      viewport_left += curpanelnrx*currentPanelScalingx*0.1;
      viewport_left += curpanelnrx*currentPanelScalingx*xsize;
      viewport_left += curpanelnrx*currentPanelScalingx*0.1;
    }
    viewport_right = viewport_left+xsize;
    if(nrpanelsx > 1) {
      viewport_right = viewport_left;
      viewport_right += currentPanelScalingx*xsize;
    }
    viewport_bottom = 0.35-0.25*ysize*ysizepa;
    if(nrpanelsy > 1) {
      currentPanelScalingy = (0.55*ysize + 0.25*ysize*ysizepa)/((nrpanelsy-1)*(0.35-0.25*ysize*ysizepa) + nrpanelsy*(0.55*ysize + 0.25*ysize*ysizepa) + (nrpanelsy-1)*(1-0.35-0.55*ysize));
      if(currentPanelScalingy < currentPanelScaling)
 currentPanelScaling = currentPanelScalingy;
      viewport_bottom += (nrpanelsy-curpanelnry-1)*currentPanelScalingy*(0.35-0.25*ysize*ysizepa);
      viewport_bottom += (nrpanelsy-curpanelnry-1)*currentPanelScalingy*(0.55*ysize + 0.25*ysize*ysizepa);
      viewport_bottom += (nrpanelsy-curpanelnry-1)*currentPanelScalingy*(1-0.35-0.55*ysize);
    }
    viewport_top = viewport_bottom + (0.55*ysize + 0.25*ysize*ysizepa);
    if(nrpanelsy > 1) {
      viewport_top = viewport_bottom;
      viewport_top += currentPanelScalingy*(0.55*ysize + 0.25*ysize*ysizepa);
    }
    if(curpanelnrx == 0 && curpanelnry == 0)
      ppgpage();
    curpanelnrx++;
    if(curpanelnrx == nrpanelsx) {
      curpanelnrx = 0;
      curpanelnry++;
    }
    if(curpanelnry == nrpanelsy)
      curpanelnry = 0;
    if(datain.NrSubints > 1 || datain.NrFreqChan > 1) {
      fflush(stdout);
      printwarning(application.verbose_state.debug, "WARNING ppolFig: Only showing polarization of the first subint/frequency channel");
    }
    pgplot_options.viewport.dxplot = viewport_left - 0.1;
    pgplot_options.viewport.xsize = (viewport_right - 0.1 - pgplot_options.viewport.dxplot)/0.8;
    pgplot_options.viewport.ysize = (viewport_top - viewport_bottom)/(0.55 + 0.25*ysizepa);
    pgplot_options.viewport.dyplot = viewport_top - 0.35 - 0.55*(viewport_top - viewport_bottom)/(0.55 + 0.25*ysizepa);
    strcpy(pgplot_options.viewport.plotDevice, PlotDevice);
    pgplot_options.viewport.noclear = 1;
    pgplot_options.viewport.dontclose = 1;
    pgplot_options.viewport.dontopen = !firstfiletoopen;
    pgplot_options.box.box_xtick = xtick;
    pgplot_options.box.box_nxsub = nxsub;
    if(titleset)
      strcpy(pgplot_options.box.title, title);
    else
      strcpy(pgplot_options.box.title, datain.filename);
    pgplot_options.box.title_ch = titlech*currentPanelScaling;
    pgplot_options.box.label_ch = labelch*currentPanelScaling;
    pgplot_options.box.box_labelsize = boxch*currentPanelScaling;
    if(padist_filename_index) {
      if(!openPSRData(&padist_data, argv[padist_filename_index], 0, 0, 1, 0, application.verbose_state))
 return 0;
      if(padist_data.gentype != GENTYPE_PADIST) {
 fflush(stdout);
 printerror(application.verbose_state.debug, "ERROR ppolFig: The file opened as a PA distribution does not have a gentype set to be a PA distribution. It is currently set to %s.", returnGenType_str(padist_data.gentype));
 return 0;
      }
      if(padist_data.NrPols != 1) {
 fflush(stdout);
 printerror(application.verbose_state.debug, "ERROR ppolFig: The file opened as a PA distribution contains %d of polarization channels. Only one channel was expected.", padist_data.NrPols);
 return 0;
      }
    }
    if(elldist_filename_index) {
      if(!openPSRData(&elldist_data, argv[elldist_filename_index], 0, 0, 1, 0, application.verbose_state))
 return 0;
      if(elldist_data.gentype != GENTYPE_ELLDIST) {
 fflush(stdout);
 printerror(application.verbose_state.debug, "ERROR ppolFig: The file opened as an ellipticity angle distribution does not have a gentype set to be a ellipticity angle distribution. It is currently set to %s.", returnGenType_str(elldist_data.gentype));
 return 0;
      }
      if(elldist_data.NrPols != 1) {
 fflush(stdout);
 printerror(application.verbose_state.debug, "ERROR ppolFig: The file opened as an ellipticity angle distribution contains %d of polarization channels. Only one channel was expected.", elldist_data.NrPols);
 return 0;
      }
    }
    datafile_definition *padist_ptr, *elldist_ptr, *spstatfrac_ptr;
    padist_ptr = NULL;
    elldist_ptr = NULL;
    spstatfrac_ptr = NULL;
    if(padist_filename_index) {
      padist_ptr = &padist_data;
    }
    if(elldist_filename_index) {
      elldist_ptr = &elldist_data;
    }
    pgplotPAplot(datain, showtotPol, nopaswing, showEll, &pgplot_options, label_x, label_I, label_pa, label_ell, "Fraction", plotl1, plotl2, xunit_type, loffset, plotI1, plotI2, plotp1, plotp2, PAoffset, sigma_limit, datalinewidth, ysizepa, dashed, noylabel, "-text", "-text_pa", ytick_pa, nysub_pa, "-text_ell", ytick_ell, nysub_ell, "-text_padist", "-text_elldist", "-text_spstat", "-herrorbar", "-herrorbarpa", "-herrorbar2", "-herrorbarell", "-herrorbarpadist", "-herrorbarelldist", "-verrorbar", "-verrorbarpa", "-verrorbar2", "-verrorbarell", argc, argv, outline, outline, outline_color, overlayPA, overlayalpha, overlaybeta, overlaypa0, overlayl0, overlayFine, nrJumps, jump_longitude, jump_offset, padist_ptr, plotp1_padist, plotp2_padist, padist_saturize, padist_overlayavpa, padist_paswing, elldist_ptr, elldist_saturize, nowedge, spstatfrac_ptr, application.verbose_state);
    if(firstfiletoopen) {
      ppgqid(&deviceID);
    }
    if(padist_filename_index) {
      closePSRData(&padist_data, 0, 0, application.verbose_state);
    }
    if(elldist_filename_index) {
      closePSRData(&elldist_data, 0, 0, application.verbose_state);
    }
    closePSRData(&datain, 0, 0, application.verbose_state);
    firstfiletoopen = 0;
  }
  ppgend();
  terminateApplication(&application);
  return 0;
}
